@@ -1,7 +1,7 @@
 use crate::{
     client::{Mailbox, SessionChannel},
     constants::CLIENT_PIPE_CAPACITY,
-    data::{Request, RequestData, ResponseData},
+    data::{PtySize, Request, RequestData, ResponseData},
     net::TransportError,
 };
 use derive_more::{Display, Error, From};
@@ -44,7 +44,7 @@ pub struct RemoteProcess {
     id: usize,
 
     /// Id used to map back to mailbox
-    pub(crate) origin_id: usize,
+    origin_id: usize,
 
     // Sender to abort req task
     abort_req_task_tx: mpsc::Sender<()>,
@@ -61,8 +61,11 @@ pub struct RemoteProcess {
     /// Receiver for stderr
     pub stderr: Option<RemoteStderr>,
 
+    /// Sender for resize events
+    resizer: RemoteProcessResizer,
+
     /// Sender for kill events
-    kill: mpsc::Sender<()>,
+    killer: RemoteProcessKiller,
 
     /// Task that waits for the process to complete
     wait_task: JoinHandle<()>,
@@ -78,7 +81,8 @@ impl RemoteProcess {
         mut channel: SessionChannel,
         cmd: impl Into<String>,
         args: Vec<String>,
-        detached: bool,
+        persist: bool,
+        pty: Option<PtySize>,
     ) -> Result<Self, RemoteProcessError> {
         let tenant = tenant.into();
         let cmd = cmd.into();
@@ -87,10 +91,11 @@ impl RemoteProcess {
         let mut mailbox = channel
             .mail(Request::new(
                 tenant.as_str(),
-                vec![RequestData::ProcRun {
+                vec![RequestData::ProcSpawn {
                     cmd,
                     args,
-                    detached,
+                    persist,
+                    pty,
                 }],
             ))
             .await?;
@@ -105,7 +110,7 @@ impl RemoteProcess {
             Some(res) => {
                 let origin_id = res.origin_id;
                 match res.payload.into_iter().next().unwrap() {
-                    ResponseData::ProcStart { id } => (id, origin_id),
+                    ResponseData::ProcSpawned { id } => (id, origin_id),
                     ResponseData::Error(x) => {
                         return Err(RemoteProcessError::TransportError(TransportError::IoError(
                             x.into(),
@@ -132,6 +137,7 @@ impl RemoteProcess {
         let (stdin_tx, stdin_rx) = mpsc::channel(CLIENT_PIPE_CAPACITY);
         let (stdout_tx, stdout_rx) = mpsc::channel(CLIENT_PIPE_CAPACITY);
         let (stderr_tx, stderr_rx) = mpsc::channel(CLIENT_PIPE_CAPACITY);
+        let (resize_tx, resize_rx) = mpsc::channel(1);
 
         // Used to terminate request task, either explicitly by the process or internally
         // by the response task when it terminates
@@ -159,7 +165,7 @@ impl RemoteProcess {
                 _ = abort_req_task_rx.recv() => {
                     panic!("killed");
                 }
-                res = process_outgoing_requests(tenant, id, channel, stdin_rx, kill_rx) => {
+                res = process_outgoing_requests(tenant, id, channel, stdin_rx, resize_rx, kill_rx) => {
                     res
                 }
             }
@@ -183,7 +189,8 @@ impl RemoteProcess {
             stdin: Some(RemoteStdin(stdin_tx)),
             stdout: Some(RemoteStdout(stdout_rx)),
             stderr: Some(RemoteStderr(stderr_rx)),
-            kill: kill_tx,
+            resizer: RemoteProcessResizer(resize_tx),
+            killer: RemoteProcessKiller(kill_tx),
             wait_task,
             status,
         })
@@ -194,6 +201,11 @@ impl RemoteProcess {
         self.id
     }
 
+    /// Returns the id of the request that spawned this process
+    pub fn origin_id(&self) -> usize {
+        self.origin_id
+    }
+
     /// Checks if the process has completed, returning the exit status if it has, without
     /// consuming the process itself. Note that this does not include join errors that can
     /// occur when aborting and instead converts any error to a status of false. To acquire
@@ -218,6 +230,26 @@ impl RemoteProcess {
             .unwrap_or(Err(RemoteProcessError::UnexpectedEof))
     }
 
+    /// Resizes the pty of the remote process if it is attached to one
+    pub async fn resize(&self, size: PtySize) -> Result<(), RemoteProcessError> {
+        self.resizer.resize(size).await
+    }
+
+    /// Clones a copy of the remote process pty resizer
+    pub fn clone_resizer(&self) -> RemoteProcessResizer {
+        self.resizer.clone()
+    }
+
+    /// Submits a kill request for the running process
+    pub async fn kill(&mut self) -> Result<(), RemoteProcessError> {
+        self.killer.kill().await
+    }
+
+    /// Clones a copy of the remote process killer
+    pub fn clone_killer(&self) -> RemoteProcessKiller {
+        self.killer.clone()
+    }
+
     /// Aborts the process by forcing its response task to shutdown, which means that a call
     /// to `wait` will return an error. Note that this does **not** send a kill request, so if
     /// you want to be nice you should send the request before aborting.
@@ -225,10 +257,31 @@ impl RemoteProcess {
         let _ = self.abort_req_task_tx.try_send(());
         let _ = self.abort_res_task_tx.try_send(());
     }
+}
 
+/// A handle to the channel to kill a remote process
+#[derive(Clone, Debug)]
+pub struct RemoteProcessResizer(mpsc::Sender<PtySize>);
+
+impl RemoteProcessResizer {
+    /// Resizes the pty of the remote process if it is attached to one
+    pub async fn resize(&self, size: PtySize) -> Result<(), RemoteProcessError> {
+        self.0
+            .send(size)
+            .await
+            .map_err(|_| RemoteProcessError::ChannelDead)?;
+        Ok(())
+    }
+}
+
+/// A handle to the channel to kill a remote process
+#[derive(Clone, Debug)]
+pub struct RemoteProcessKiller(mpsc::Sender<()>);
+
+impl RemoteProcessKiller {
     /// Submits a kill request for the running process
     pub async fn kill(&mut self) -> Result<(), RemoteProcessError> {
-        self.kill
+        self.0
             .send(())
             .await
             .map_err(|_| RemoteProcessError::ChannelDead)?;
@@ -237,14 +290,19 @@ impl RemoteProcess {
 }
 
 /// A handle to a remote process' standard input (stdin)
-#[derive(Debug)]
-pub struct RemoteStdin(mpsc::Sender<String>);
+#[derive(Clone, Debug)]
+pub struct RemoteStdin(mpsc::Sender<Vec<u8>>);
 
 impl RemoteStdin {
+    /// Creates a disconnected remote stdin
+    pub fn disconnected() -> Self {
+        Self(mpsc::channel(1).0)
+    }
+
     /// Tries to write to the stdin of the remote process, returning ok if immediately
     /// successful, `WouldBlock` if would need to wait to send data, and `BrokenPipe`
     /// if stdin has been closed
-    pub fn try_write(&mut self, data: impl Into<String>) -> io::Result<()> {
+    pub fn try_write(&mut self, data: impl Into<Vec<u8>>) -> io::Result<()> {
         match self.0.try_send(data.into()) {
             Ok(data) => Ok(data),
             Err(TrySendError::Full(_)) => Err(io::Error::from(io::ErrorKind::WouldBlock)),
@@ -252,14 +310,24 @@ impl RemoteStdin {
         }
     }
 
+    /// Same as `try_write`, but with a string
+    pub fn try_write_str(&mut self, data: impl Into<String>) -> io::Result<()> {
+        self.try_write(data.into().into_bytes())
+    }
+
     /// Writes data to the stdin of a specific remote process
-    pub async fn write(&mut self, data: impl Into<String>) -> io::Result<()> {
+    pub async fn write(&mut self, data: impl Into<Vec<u8>>) -> io::Result<()> {
         self.0
             .send(data.into())
             .await
             .map_err(|x| io::Error::new(io::ErrorKind::BrokenPipe, x))
     }
 
+    /// Same as `write`, but with a string
+    pub async fn write_str(&mut self, data: impl Into<String>) -> io::Result<()> {
+        self.write(data.into().into_bytes()).await
+    }
+
     /// Checks if stdin has been closed
     pub fn is_closed(&self) -> bool {
         self.0.is_closed()
@@ -268,12 +336,12 @@ impl RemoteStdin {
 
 /// A handle to a remote process' standard output (stdout)
 #[derive(Debug)]
-pub struct RemoteStdout(mpsc::Receiver<String>);
+pub struct RemoteStdout(mpsc::Receiver<Vec<u8>>);
 
 impl RemoteStdout {
     /// Tries to receive latest stdout for a remote process, yielding `None`
     /// if no stdout is available, and `BrokenPipe` if stdout has been closed
-    pub fn try_read(&mut self) -> io::Result<Option<String>> {
+    pub fn try_read(&mut self) -> io::Result<Option<Vec<u8>>> {
         match self.0.try_recv() {
             Ok(data) => Ok(Some(data)),
             Err(TryRecvError::Empty) => Ok(None),
@@ -281,24 +349,41 @@ impl RemoteStdout {
         }
     }
 
+    /// Same as `try_read`, but returns a string
+    pub fn try_read_string(&mut self) -> io::Result<Option<String>> {
+        self.try_read().and_then(|x| match x {
+            Some(data) => String::from_utf8(data)
+                .map(Some)
+                .map_err(|x| io::Error::new(io::ErrorKind::InvalidData, x)),
+            None => Ok(None),
+        })
+    }
+
     /// Retrieves the latest stdout for a specific remote process, and `BrokenPipe` if stdout has
     /// been closed
-    pub async fn read(&mut self) -> io::Result<String> {
+    pub async fn read(&mut self) -> io::Result<Vec<u8>> {
         self.0
             .recv()
             .await
             .ok_or_else(|| io::Error::from(io::ErrorKind::BrokenPipe))
     }
+
+    /// Same as `read`, but returns a string
+    pub async fn read_string(&mut self) -> io::Result<String> {
+        self.read().await.and_then(|data| {
+            String::from_utf8(data).map_err(|x| io::Error::new(io::ErrorKind::InvalidData, x))
+        })
+    }
 }
 
 /// A handle to a remote process' stderr
 #[derive(Debug)]
-pub struct RemoteStderr(mpsc::Receiver<String>);
+pub struct RemoteStderr(mpsc::Receiver<Vec<u8>>);
 
 impl RemoteStderr {
     /// Tries to receive latest stderr for a remote process, yielding `None`
     /// if no stderr is available, and `BrokenPipe` if stderr has been closed
-    pub fn try_read(&mut self) -> io::Result<Option<String>> {
+    pub fn try_read(&mut self) -> io::Result<Option<Vec<u8>>> {
         match self.0.try_recv() {
             Ok(data) => Ok(Some(data)),
             Err(TryRecvError::Empty) => Ok(None),
@@ -306,14 +391,31 @@ impl RemoteStderr {
         }
     }
 
+    /// Same as `try_read`, but returns a string
+    pub fn try_read_string(&mut self) -> io::Result<Option<String>> {
+        self.try_read().and_then(|x| match x {
+            Some(data) => String::from_utf8(data)
+                .map(Some)
+                .map_err(|x| io::Error::new(io::ErrorKind::InvalidData, x)),
+            None => Ok(None),
+        })
+    }
+
     /// Retrieves the latest stderr for a specific remote process, and `BrokenPipe` if stderr has
     /// been closed
-    pub async fn read(&mut self) -> io::Result<String> {
+    pub async fn read(&mut self) -> io::Result<Vec<u8>> {
         self.0
             .recv()
             .await
             .ok_or_else(|| io::Error::from(io::ErrorKind::BrokenPipe))
     }
+
+    /// Same as `read`, but returns a string
+    pub async fn read_string(&mut self) -> io::Result<String> {
+        self.read().await.and_then(|data| {
+            String::from_utf8(data).map_err(|x| io::Error::new(io::ErrorKind::InvalidData, x))
+        })
+    }
 }
 
 /// Helper function that loops, processing outgoing stdin requests to a remote process as well as
@@ -322,7 +424,8 @@ async fn process_outgoing_requests(
     tenant: String,
     id: usize,
     mut channel: SessionChannel,
-    mut stdin_rx: mpsc::Receiver<String>,
+    mut stdin_rx: mpsc::Receiver<Vec<u8>>,
+    mut resize_rx: mpsc::Receiver<PtySize>,
     mut kill_rx: mpsc::Receiver<()>,
 ) -> Result<(), RemoteProcessError> {
     let result = loop {
@@ -338,6 +441,17 @@ async fn process_outgoing_requests(
                     None => break Err(RemoteProcessError::ChannelDead),
                 }
             }
+            size = resize_rx.recv() => {
+                match size {
+                    Some(size) => channel.fire(
+                        Request::new(
+                            tenant.as_str(),
+                            vec![RequestData::ProcResizePty { id, size }]
+                        )
+                    ).await?,
+                    None => break Err(RemoteProcessError::ChannelDead),
+                }
+            }
             msg = kill_rx.recv() => {
                 if msg.is_some() {
                     channel.fire(Request::new(
@@ -360,8 +474,8 @@ async fn process_outgoing_requests(
 async fn process_incoming_responses(
     proc_id: usize,
     mut mailbox: Mailbox,
-    stdout_tx: mpsc::Sender<String>,
-    stderr_tx: mpsc::Sender<String>,
+    stdout_tx: mpsc::Sender<Vec<u8>>,
+    stderr_tx: mpsc::Sender<Vec<u8>>,
     kill_tx: mpsc::Sender<()>,
 ) -> Result<(bool, Option<i32>), RemoteProcessError> {
     while let Some(res) = mailbox.next().await {
@@ -431,6 +545,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -470,6 +585,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -516,6 +632,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -529,7 +646,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
@@ -562,6 +679,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -575,7 +693,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
@@ -619,6 +737,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -632,7 +751,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
@@ -656,7 +775,7 @@ mod tests {
         {
             RequestData::ProcStdin { id, data } => {
                 assert_eq!(*id, 12345);
-                assert_eq!(data, "some input");
+                assert_eq!(data, b"some input");
             }
             x => panic!("Unexpected request: {:?}", x),
         }
@@ -675,6 +794,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -688,7 +808,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
@@ -702,14 +822,14 @@ mod tests {
                 req.id,
                 vec![ResponseData::ProcStdout {
                     id,
-                    data: String::from("some out"),
+                    data: b"some out".to_vec(),
                 }],
             ))
             .await
             .unwrap();
 
         let out = proc.stdout.as_mut().unwrap().read().await.unwrap();
-        assert_eq!(out, "some out");
+        assert_eq!(out, b"some out");
     }
 
     #[tokio::test]
@@ -725,6 +845,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -738,7 +859,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
@@ -752,14 +873,14 @@ mod tests {
                 req.id,
                 vec![ResponseData::ProcStderr {
                     id,
-                    data: String::from("some err"),
+                    data: b"some err".to_vec(),
                 }],
             ))
             .await
             .unwrap();
 
         let out = proc.stderr.as_mut().unwrap().read().await.unwrap();
-        assert_eq!(out, "some err");
+        assert_eq!(out, b"some err");
     }
 
     #[tokio::test]
@@ -775,6 +896,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -788,7 +910,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
@@ -813,6 +935,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -826,7 +949,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
@@ -859,6 +982,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -872,7 +996,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
@@ -914,6 +1038,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -927,7 +1052,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
@@ -957,6 +1082,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -970,7 +1096,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
@@ -1007,6 +1133,7 @@ mod tests {
                 String::from("cmd"),
                 vec![String::from("arg")],
                 false,
+                None,
             )
             .await
         });
@@ -1020,7 +1147,7 @@ mod tests {
             .send(Response::new(
                 "test-tenant",
                 req.id,
-                vec![ResponseData::ProcStart { id }],
+                vec![ResponseData::ProcSpawned { id }],
             ))
             .await
             .unwrap();
