@@ -1,6 +1,12 @@
 use crate::{
-    client::{RemoteLspProcess, RemoteProcess, RemoteProcessError, SessionChannel},
-    data::{DirEntry, Error as Failure, Metadata, Request, RequestData, ResponseData, SystemInfo},
+    client::{
+        RemoteLspProcess, RemoteProcess, RemoteProcessError, SessionChannel, UnwatchError,
+        WatchError, Watcher,
+    },
+    data::{
+        ChangeKindSet, DirEntry, Error as Failure, Metadata, PtySize, Request, RequestData,
+        ResponseData, SystemInfo,
+    },
     net::TransportError,
 };
 use derive_more::{Display, Error, From};
@@ -115,13 +121,31 @@ pub trait SessionChannelExt {
         dst: impl Into<PathBuf>,
     ) -> AsyncReturn<'_, ()>;
 
+    /// Watches a remote file or directory
+    fn watch(
+        &mut self,
+        tenant: impl Into<String>,
+        path: impl Into<PathBuf>,
+        recursive: bool,
+        only: impl Into<ChangeKindSet>,
+        except: impl Into<ChangeKindSet>,
+    ) -> AsyncReturn<'_, Watcher, WatchError>;
+
+    /// Unwatches a remote file or directory
+    fn unwatch(
+        &mut self,
+        tenant: impl Into<String>,
+        path: impl Into<PathBuf>,
+    ) -> AsyncReturn<'_, (), UnwatchError>;
+
     /// Spawns a process on the remote machine
     fn spawn(
         &mut self,
         tenant: impl Into<String>,
         cmd: impl Into<String>,
         args: Vec<impl Into<String>>,
-        detached: bool,
+        persist: bool,
+        pty: Option<PtySize>,
     ) -> AsyncReturn<'_, RemoteProcess, RemoteProcessError>;
 
     /// Spawns an LSP process on the remote machine
@@ -130,7 +154,8 @@ pub trait SessionChannelExt {
         tenant: impl Into<String>,
         cmd: impl Into<String>,
         args: Vec<impl Into<String>>,
-        detached: bool,
+        persist: bool,
+        pty: Option<PtySize>,
     ) -> AsyncReturn<'_, RemoteLspProcess, RemoteProcessError>;
 
     /// Retrieves information about the remote system
@@ -250,7 +275,7 @@ impl SessionChannelExt for SessionChannel {
             tenant,
             RequestData::Exists { path: path.into() },
             |data| match data {
-                ResponseData::Exists(x) => Ok(x),
+                ResponseData::Exists { value } => Ok(value),
                 ResponseData::Error(x) => Err(SessionChannelExtError::Failure(x)),
                 _ => Err(SessionChannelExtError::MismatchedResponse),
             }
@@ -369,19 +394,65 @@ impl SessionChannelExt for SessionChannel {
         )
     }
 
+    fn watch(
+        &mut self,
+        tenant: impl Into<String>,
+        path: impl Into<PathBuf>,
+        recursive: bool,
+        only: impl Into<ChangeKindSet>,
+        except: impl Into<ChangeKindSet>,
+    ) -> AsyncReturn<'_, Watcher, WatchError> {
+        let tenant = tenant.into();
+        let path = path.into();
+        let only = only.into();
+        let except = except.into();
+        Box::pin(async move {
+            Watcher::watch(tenant, self.clone(), path, recursive, only, except).await
+        })
+    }
+
+    fn unwatch(
+        &mut self,
+        tenant: impl Into<String>,
+        path: impl Into<PathBuf>,
+    ) -> AsyncReturn<'_, (), UnwatchError> {
+        fn inner_unwatch(
+            channel: &mut SessionChannel,
+            tenant: impl Into<String>,
+            path: impl Into<PathBuf>,
+        ) -> AsyncReturn<'_, ()> {
+            make_body!(
+                channel,
+                tenant,
+                RequestData::Unwatch { path: path.into() },
+                @ok
+            )
+        }
+
+        let tenant = tenant.into();
+        let path = path.into();
+
+        Box::pin(async move {
+            inner_unwatch(self, tenant, path)
+                .await
+                .map_err(UnwatchError::from)
+        })
+    }
+
     fn spawn(
         &mut self,
         tenant: impl Into<String>,
         cmd: impl Into<String>,
         args: Vec<impl Into<String>>,
-        detached: bool,
+        persist: bool,
+        pty: Option<PtySize>,
     ) -> AsyncReturn<'_, RemoteProcess, RemoteProcessError> {
         let tenant = tenant.into();
         let cmd = cmd.into();
         let args = args.into_iter().map(Into::into).collect();
-        Box::pin(
-            async move { RemoteProcess::spawn(tenant, self.clone(), cmd, args, detached).await },
-        )
+        Box::pin(async move {
+            RemoteProcess::spawn(tenant, self.clone(), cmd, args, persist, pty).await
+        })
     }
 
     fn spawn_lsp(
@@ -389,14 +460,15 @@ impl SessionChannelExt for SessionChannel {
         tenant: impl Into<String>,
         cmd: impl Into<String>,
         args: Vec<impl Into<String>>,
-        detached: bool,
+        persist: bool,
+        pty: Option<PtySize>,
     ) -> AsyncReturn<'_, RemoteLspProcess, RemoteProcessError> {
         let tenant = tenant.into();
         let cmd = cmd.into();
         let args = args.into_iter().map(Into::into).collect();
-        Box::pin(
-            async move { RemoteLspProcess::spawn(tenant, self.clone(), cmd, args, detached).await },
-        )
+        Box::pin(async move {
+            RemoteLspProcess::spawn(tenant, self.clone(), cmd, args, persist, pty).await
+        })
     }
 
     fn system_info(&mut self, tenant: impl Into<String>) -> AsyncReturn<'_, SystemInfo> {
