@@ -43,6 +43,9 @@ pub enum SessionDetails {
 
     /// Indicates session type is inmemory
     Inmemory { tag: String },
+
+    /// Indicates session type is a custom type (such as ssh)
+    Custom { tag: String },
 }
 
 impl SessionDetails {
@@ -52,6 +55,7 @@ impl SessionDetails {
             Self::Tcp { tag, .. } => tag.as_str(),
             Self::Socket { tag, .. } => tag.as_str(),
             Self::Inmemory { tag } => tag.as_str(),
+            Self::Custom { tag } => tag.as_str(),
         }
     }
 
@@ -124,6 +128,11 @@ impl Session {
             .await
             .and_then(convert::identity)
     }
+
+    /// Convert into underlying channel
+    pub fn into_channel(self) -> SessionChannel {
+        self.channel
+    }
 }
 
 #[cfg(unix)]
@@ -213,7 +222,7 @@ impl Session {
                         break;
                     }
                     Err(x) => {
-                        error!("{}", x);
+                        error!("Failed to receive response from server: {}", x);
                         break;
                     }
                 }
