@@ -21,11 +21,30 @@ mod unix;
 #[cfg(unix)]
 pub use unix::*;
 
+#[derive(Debug, Display, Error, From)]
+pub struct SerializeError(#[error(not(source))] String);
+
+#[derive(Debug, Display, Error, From)]
+pub struct DeserializeError(#[error(not(source))] String);
+
+fn serialize_to_vec<T: Serialize>(value: &T) -> Result<Vec<u8>, SerializeError> {
+    let mut v = Vec::new();
+
+    let _ = ciborium::ser::into_writer(value, &mut v).map_err(|x| SerializeError(x.to_string()))?;
+
+    Ok(v)
+}
+
+fn deserialize_from_slice<T: DeserializeOwned>(slice: &[u8]) -> Result<T, DeserializeError> {
+    ciborium::de::from_reader(slice).map_err(|x| DeserializeError(x.to_string()))
+}
+
 #[derive(Debug, Display, Error, From)]
 pub enum TransportError {
     CryptoError(SecretKeyError),
     IoError(io::Error),
-    SerializeError(serde_cbor::Error),
+    SerializeError(SerializeError),
+    DeserializeError(DeserializeError),
 }
 
 /// Interface representing a two-way data stream
@@ -63,7 +82,7 @@ where
     pub async fn send<D: Serialize>(&mut self, data: D) -> Result<(), TransportError> {
         // Serialize data into a byte stream
         // NOTE: Cannot used packed implementation for now due to issues with deserialization
-        let data = serde_cbor::to_vec(&data)?;
+        let data = serialize_to_vec(&data)?;
 
         // Use underlying codec to send data (may encrypt, sign, etc.)
         self.0.send(&data).await.map_err(TransportError::from)
@@ -77,7 +96,7 @@ where
             let data = data?;
 
             // Deserialize byte stream into our expected type
-            let data = serde_cbor::from_slice(&data)?;
+            let data = deserialize_from_slice(&data)?;
             Ok(Some(data))
         } else {
             Ok(None)
@@ -153,7 +172,7 @@ where
     pub async fn send<D: Serialize>(&mut self, data: D) -> Result<(), TransportError> {
         // Serialize data into a byte stream
         // NOTE: Cannot used packed implementation for now due to issues with deserialization
-        let data = serde_cbor::to_vec(&data)?;
+        let data = serialize_to_vec(&data)?;
 
         // Use underlying codec to send data (may encrypt, sign, etc.)
         self.0.send(&data).await.map_err(TransportError::from)
@@ -179,7 +198,7 @@ where
             let data = data?;
 
             // Deserialize byte stream into our expected type
-            let data = serde_cbor::from_slice(&data)?;
+            let data = deserialize_from_slice(&data)?;
             Ok(Some(data))
         } else {
             Ok(None)
@@ -220,7 +239,7 @@ mod tests {
             value: 123,
         };
 
-        let bytes = serde_cbor::to_vec(&data).unwrap();
+        let bytes = serialize_to_vec(&data).unwrap();
         let len = (bytes.len() as u64).to_be_bytes();
         let mut frame = Vec::new();
         frame.extend(len.iter().copied());
@@ -253,7 +272,7 @@ mod tests {
         struct OtherTestData(usize);
 
         let data = OtherTestData(123);
-        let bytes = serde_cbor::to_vec(&data).unwrap();
+        let bytes = serialize_to_vec(&data).unwrap();
         let len = (bytes.len() as u64).to_be_bytes();
         let mut frame = Vec::new();
         frame.extend(len.iter().copied());
@@ -262,7 +281,7 @@ mod tests {
         tx.send(frame).await.unwrap();
         let result = transport.receive::<TestData>().await;
         match result {
-            Err(TransportError::SerializeError(_)) => {}
+            Err(TransportError::DeserializeError(_)) => {}
             x => panic!("Unexpected result: {:?}", x),
         }
     }
@@ -277,7 +296,7 @@ mod tests {
             value: 123,
         };
 
-        let bytes = serde_cbor::to_vec(&data).unwrap();
+        let bytes = serialize_to_vec(&data).unwrap();
         let len = (bytes.len() as u64).to_be_bytes();
         let mut frame = Vec::new();
         frame.extend(len.iter().copied());
@@ -314,7 +333,7 @@ mod tests {
             struct OtherTestData(usize);
 
             let data = OtherTestData(123);
-            let bytes = serde_cbor::to_vec(&data).unwrap();
+            let bytes = serialize_to_vec(&data).unwrap();
             let len = (bytes.len() as u64).to_be_bytes();
             let mut frame = Vec::new();
             frame.extend(len.iter().copied());
@@ -323,7 +342,7 @@ mod tests {
             tx.send(frame).await.unwrap();
             let result = rh.receive::<TestData>().await;
             match result {
-                Err(TransportError::SerializeError(_)) => {}
+                Err(TransportError::DeserializeError(_)) => {}
                 x => panic!("Unexpected result: {:?}", x),
             }
         }
@@ -339,7 +358,7 @@ mod tests {
                 value: 123,
             };
 
-            let bytes = serde_cbor::to_vec(&data).unwrap();
+            let bytes = serialize_to_vec(&data).unwrap();
             let len = (bytes.len() as u64).to_be_bytes();
             let mut frame = Vec::new();
             frame.extend(len.iter().copied());
@@ -365,7 +384,7 @@ mod tests {
                 value: 123,
             };
 
-            let bytes = serde_cbor::to_vec(&data).unwrap();
+            let bytes = serialize_to_vec(&data).unwrap();
             let len = (bytes.len() as u64).to_be_bytes();
             let mut frame = Vec::new();
             frame.extend(len.iter().copied());
