@@ -1,6 +1,8 @@
 mod handler;
+mod process;
 mod state;
 
+pub(crate) use process::{InputChannel, ProcessKiller, ProcessPty};
 use state::State;
 
 use crate::{
